"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/history.py $
$Id: history.py 23447 2004-02-06 17:36:45Z dbinger $
"""

from mx import DateTime
from dulcinea.base import DulcineaBase, DulcineaPersistent
from dulcinea.typeutils import typecheck
from dulcinea.user import DulcineaUser

class Event (DulcineaBase):
    """A single event in a history log.  A History instance is mainly a
    list of Event instances. Each event has a user ID, a timestamp, an
    "event code" (short string giving the general class of event) and a
    message giving details about the event.

    Instance attributes:
      timestamp : DateTime
      user : DulcineaUser
      event_code : string
      message : string

    Feel free to access these directly; do *not* modify them!  (The
    whole point of Event is to be immutable.)

    Also, Event objects should never be created by outside code:
    always go through the 'add_event()' method of History.
    """

    timestamp_is = DateTime.DateTimeType
    user_is = DulcineaUser
    event_code_is = str
    message_is = str

    def __init__(self, user, event_code, message, timestamp=None):
        typecheck(timestamp, DateTime.DateTimeType, allow_none=1)
        if timestamp is None:
            self.timestamp = DateTime.now()
        else:
            self.timestamp = timestamp
        typecheck(user, DulcineaUser, allow_none=1)
        self.user = user
        self.event_code = event_code
        self.message = message

    def __str__(self):
        return "%s %s" % (self.timestamp, self.event_code)

    def __cmp__(self, other):
        # DateTime objects can't be compared to None
        if self.timestamp is None and other.timestamp is not None:
            return -1
        if other.timestamp is None and self.timestamp is not None:
            return +1

        # Compare all attributes in turn, but stop as soon as cmp()
        # returns true (non-zero, ie. non-equal) for an attribute.
        return (cmp(self.timestamp, other.timestamp) or
                cmp(self.event_code, other.event_code) or
                cmp(self.user, other.user) or
                cmp(self.message, other.message))

    def get_timestamp(self):
        return self.timestamp

    def get_user(self):
        return self.user

    def get_event_code(self):
        return self.event_code

    def get_message(self):
        return self.message


class History (DulcineaPersistent):
    """A list of history events attached to a single object.

    Instance attributes:
      events : [Event]
        the list of events that comprise the object's history
    """

    events_is = [Event]

    def __init__(self):
        self.events = []

    def __str__(self):
        return "history (%d events)" % len(self.events)

    def __len__(self):
        return len(self.events)

    def __getitem__(self, i):
        return self.events[i]

    def insert_event(self, event):
        i = 0
        for e in self.events:
            if e > event:
                break
            i += 1
        self.events.insert(i, event)
        self._p_changed = 1

    def add_event(self, user, event_code, message, timestamp=None):
        """(user : DulcineaUser, event_code : string, message : string,
            timestamp : DateTime = None)

        Create a new Event instance. If timestamp is None (the default),
        the new event is stamped with the current time and appended to
        the end of the history list. If timestamp is supplied, it is
        used as the event's timestamp, and the event is inserted into
        the history list so that its ordering by timestamp is preserved.
        """
        new_event = Event(user, event_code, message, timestamp)
        if timestamp is None:
            self.events.append(new_event)
            self._p_changed = 1
        else:
            self.insert_event(new_event)

    def add_unique_event(self, user, event_code, message):
        """(user : DulcineaUser, event_code : string, message : string)

        Create a new Event instance. If the last event on the list had
        the same event code and user, replace the last event with the
        new one.
        """
        new_event = Event(user, event_code, message, timestamp=None)
        replace_event = False
        if self.events:
            last = self.events[-1]
            if (last.user is new_event.user and
                last.event_code == new_event.event_code):
                replace_event = True

        if replace_event:
            self.events[-1] = new_event
        else:
            self.events.append(new_event)
        self._p_changed = 1

    def find_event(self, user=None, event_code=None, event_codes=None,
                   latest=False, message=None):
        """(user : DulcineaUser = None,
            event_code : string = None,
            latest : bool = False,
            message : string = None) -> Event

        Find the earliest(or latest, if latest is true) event
        matching 'user' and/or 'event_code'.  If either search criterion
        is not supplied or None, it does not affect the search, thus the
        default behaviour is to return the first event in the history
        list.  Return None if no events match.
        """
        typecheck(user, DulcineaUser, allow_none=1)
        if latest:
            indices = xrange(len(self.events)-1, -1, -1)
        else:
            indices = xrange(len(self.events))
        for i in indices:
            event = self.events[i]
            if user is not None and event.user is not user:
                continue
            if event_code is not None and event.event_code != event_code:
                continue
            if event_codes is not None and event.event_code not in event_codes:
                continue
            if message is not None and event.message != message:
                continue
            return event
        else:
            return None

    def find_events_after(self, time, event_code=None):
        return [event
                for event in self.events
                if event.get_timestamp() > time and (
                    event_code is None or
                    event.get_event_code() == event_code)]

    def find_last_date(self, event_code):
        """(event_code:str) -> DateTime | None"""
        event = self.find_event(event_code=event_code, latest=True)
        return event and event.get_timestamp()

class Historical:
    """maintains a list of history events.

    Instance attributes:
      history : History
        the list of history events that comprise the object's history

    """

    history_is = History

    def __init__(self):
        self.history = History()

    def get_history(self):
        return self.history



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_typed_list.py $
$Id: test_typed_list.py 23443 2004-02-06 15:23:48Z dbinger $

Test for dulcinea.typed_list.
"""

from types import *
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.typed_list import TypedList

class Value:

    def __init__ (self, name=None):
        if name is not None:
            self.name = name

    def get_name(self):
        return getattr(self, 'name', 'no')

class OtherValue:

    def __init__ (self, name):
        self.name = name


tested_modules = [ "dulcinea.typed_list" ]

class TestTypedList(TestScenario):

    test_cases = [ 'add', 'format', 'getset_list', 'getset_item',
                   'merge', 'find_index' ]

    def check_add(self):
        lst = TypedList() ; lst.required_class = Value
        v1 = Value("n1")
        v2 = Value("n2")
        self.test_stmt("lst.add(v1)")
        self.test_stmt("lst.add(v2)")
        self.test_val("lst.find('n1')", v1, match_ident=1)
        self.test_val("lst.find('foo')", None)
        self.test_val("len(lst)", 2)

        # adding something already on the list should raise a ValueError:
        self.test_exc("lst.add(v1)", ValueError)
        self.test_seq("lst.get_names()", ["n1", "n2"])

        # turning off type checking, (if you already know the types are ok)
        self.test_stmt("TypedList(lst.get_list(), check_types=0)")

        # check remove method, while we're at it:
        self.test_val("lst.remove('n2')", v2, match_ident=1)
        self.test_val("lst.find('n2')", None)
        self.test_val("lst.remove('foo')", None)

        ov1 = OtherValue("n1")
        ov2 = OtherValue("foo")
        self.test_exc("lst.add(ov1)", TypeError)
        self.test_exc("lst.add(ov2)", TypeError)


    def check_format(self):
        lst = TypedList()
        self.test_val("str(lst)", "no values")
        lst.add(Value("n1"))
        self.test_val("str(lst)", "1 value")
        lst.add(Value("n2"))
        self.test_val("str(lst)", "2 values")


    def check_getset_list(self):
        lst = TypedList()
        v1 = Value("n1")
        v2 = Value("n2")
        self.test_seq("lst.get_list()", [])
        lst.add(v1)
        lst.add(v2)
        self.test_val("lst.get_list()", [v1, v2])
        lst.remove(v1.name)
        self.test_val("lst.get_list()", [v2])
        self.test_stmt("lst.set_list([v1, v2])")
        self.test_val("lst.find('n2')", v2, match_ident=1)
        self.test_val("lst.get_list()", [v1, v2])
        self.test_exc("lst.set_list([v1, v2, v1])", ValueError)

    def check_getset_item(self):
        lst = TypedList() ; lst.required_class = Value
        v1 = Value("n1")
        v2 = Value("n2")
        v3 = Value("n3")
        ov1 = OtherValue("n1")

        lst.add(v1) ; lst.add(v2)
        self.test_val("len(lst)", 2)
        self.test_val("lst[0]", v1, match_ident=1)
        self.test_val("lst[1]", v2, match_ident=1)
        self.test_stmt("lst[1] = v2")
        self.test_val("lst[1]", v2, match_ident=1)
        self.test_exc("lst[1] = v1", ValueError)    # name conflict

        self.test_val("lst.remove('n1')", v1, match_ident=1)
        self.test_val("len(lst)", 1)
        self.test_stmt("lst[0] = v2")
        self.test_val("lst[0]", v2, match_ident=1)
        self.test_stmt("lst[0] = v1")
        self.test_val("lst[0]", v1, match_ident=1)
        self.test_exc("lst[1]", IndexError)

        self.test_exc("lst[0] = ov1", TypeError)
        self.test_val("lst[0]", v1, match_ident=1)

    def check_merge(self):
        lst = TypedList()
        lst2 = TypedList()

        v1 = Value("n1")
        v2 = Value("n2")
        v3 = Value("n3")
        v4 = Value("n4")

        self.test_stmt("lst.add(v1)")
        self.test_stmt("lst.add(v2)")
        self.test_stmt("lst2.add(v3)")
        self.test_stmt("lst2.add(v4)")

        self.test_seq("lst.get_names()", ['n1', 'n2'])
        # presently, lst and lst2 are mutually exclusive.
        # merge them into lst3
        self.test_stmt("lst.merge(lst2)")
        self.test_seq("lst.get_names()", ['n1', 'n2', 'n3', 'n4'])

        # merging a parameter list with itself --setting various merge flags:
        self.test_exc("lst.merge(lst)", ValueError)
        # however, setting the merge flag to ignore should leave the original
        # parameter list unchanged:
        self.test_stmt("lst.merge(lst, 'ignore')")
        self.test_seq("lst.get_names()", ['n1', 'n2', 'n3', 'n4'])

        # verify that 'replace' behavior is correct:
        v1 = Value("n1")
        lst3 = TypedList([v1])
        # first, ensure that the original list contains the param by name:
        self.test_stmt("lst.merge(lst3, 'replace')")
        self.test_seq("lst.get_list()", [v1, v2, v3, v4])
        # next verify that the 'duration' parameter has been replaced:
        self.test_val("lst.find('n1')", v1, match_ident=1)

    def check_find_index(self):
        v1 = Value("n1")
        v2 = Value("n2")
        lst = TypedList([v1, v2])

        self.test_val("lst.find_index('n1')", 0)
        self.test_val("lst.find_index('n2')", 1)
        self.test_val("lst.find_index('foo')", None)


    def check_coverage(self):
        v1 = Value("n1")
        v2 = Value("n2")
        v3 = Value()
        t1 = TypedList([v1])
        self.test_stmt("t2=t1.copy()")
        self.test_stmt("t2.add(v2)")
        self.test_val("t1.get_list()", [v1])
        self.test_val("len(t2)", 2)
        t3 = TypedList([v3])
        self.test_val("len(t1 + t3 + [1])", 3)

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/address.py $
$Id: address.py 23424 2004-02-05 16:28:17Z nascheme $

Address-related classes, functions, and constants.
"""
import re

from dulcinea.base import DulcineaPersistent
from dulcinea.country import get_country_name
from dulcinea.typeutils import typecheck
from dulcinea.spec import nspec

def is_valid_phone_number(phone_number):
    """(phone_number:str) -> bool
    """
    typecheck(phone_number, str, allow_none=1)
    if not phone_number:
        return False
    number_of_digits = 0
    for token in phone_number:
        if token in '()-.+EeXxTt ':
            continue
        if not token.isdigit():
            return False
        else:
            number_of_digits += 1
    if number_of_digits < 10:
        return False
    return True

class Address (DulcineaPersistent):
    """A mailing address, which consists of a bunch of attributes.
    You can also check the validity of an address.

    Instance attributes:
      contact_name : string
      contact_phone_number : string
      company_name : string
      street1 : string
        1st line of the street address
      street2 : string
        2nd line of the street address
      city : string
      state : string
      zip : string
      country_code : string
        ISO-3166 two-letter country code (see dulcinea.country for
        tools to map between country code and country name)
    """

    STATES = [
        "AL", "AK", "AR", "AZ", "CA", "CO", "CT", "DE", "FL", "GA",
        "HI", "IA", "ID", "IL", "IN", "KS", "KY", "LA", "MA", "MD",
        "ME", "MI", "MN", "MO", "MS", "MT", "NC", "ND", "NE", "NH",
        "NJ", "NM", "NV", "NY", "OH", "OK", "OR", "PA", "RI", "SC",
        "SD", "TN", "TX", "UT", "VA", "VT", "WA", "WI", "WV", "WY",

        # Commonwealths & territories
        "AS", "DC", "FM", "GU", "MP", "PR", "VI",

        # Armed forces codes
        "AA", "AE", "AP"]

    FRIENDLY_ATTR = { 'street1': "street address",
                      'street2': "street address",
                      'state': "state/province",
                      'zip': "postal/ZIP code",
                      'country_code': "country",
                      'contact_name': "name",
                      'company_name': "company",
                      'contact_phone_number': "phone number"}

    def __init__(self, street1=None, street2=None, city=None,
                 state=None, zip=None, country_code=None):
        """(street1:str=None, street2:str=None, city:str=None, state:str=None,
            zip:str=None, country_code:str=None)
        """
        self.contact_name = None
        self.contact_phone_number = None
        self.company_name = None
        self.set_street(street1, street2)
        self.set_city(city)
        self.set_state(state)
        self.set_zip(zip)
        self.set_country_code(country_code)

    # This should really be __eq__ -- lt/gt/etc don't make sense here.
    def __cmp__(self, other):
        # Return the first non-zero cmp()
        return (cmp(self.contact_name, other.contact_name) or
                cmp(self.contact_phone_number, other.contact_phone_number) or
                cmp(self.company_name, other.company_name) or
                cmp(self.street1, other.street1) or
                cmp(self.street2, other.street2) or
                cmp(self.city, other.city) or
                cmp(self.state, other.state) or
                cmp(self.zip, other.zip) or
                cmp(self.country_code, other.country_code))

    def get_friendly_attr(self, attr):
        """(attr : str) -> str

        Expand the attribute names returned by check_complete()
        to human-readable form.
        """
        if not hasattr(self, attr):
            raise AttributeError, attr
        fr_attr = self.FRIENDLY_ATTR.get(attr)
        if fr_attr is None:
            return " ".join(attr.split("_"))
        else:
            return fr_attr

    def is_empty(self):
        """() -> bool

        Return true if this address is empty.
        """
        values = filter(None, self.__dict__.values())
        return len(values) == 0

    def is_valid(self):
        """() -> bool

        Return true if this address is valid: either it's empty or
        it's complete (and correct!).
        """
        return self.check_complete() == []

    def check_complete(self):
        """() -> [(attrname:str, errtype:str)]

        Check whether this address is complete: must have at least one
        line of street address, city, and country; if country is US,
        must have a valid state and ZIP code too.

        Returns a list of (attrname, errtype) 2-tuples explaining any
        problems.  'errtype' is either "missing" or "bad".
        """
        errors = []

        # Check that the contact_name, contact_phone_number
        # and various critical fields in the address aren't blank.
        for field in ['contact_name', 'contact_phone_number',
                      'street1', 'city', 'country_code']:
            if not getattr(self, field):
                errors.append((field, "missing"))

        # US-specific checks
        if self.country_code == 'US':
            if not self.state:
                errors.append(('state', "missing"))
            else:
                state = self.state.upper()
                if state not in self.STATES:
                    errors.append(('state', "bad"))
                else:
                    self.state = state  # ensure uppercase

            # Check the Zip code (5 digits, or 5+4 format)
            if not self.zip:
                errors.append(('zip', "missing"))
            elif re.match(r'\d{5}(-\d{4})?$', self.zip) is None:
                errors.append(('zip', "bad"))

            # Check Phone number (10 digits, some special chars ok)
            if self.contact_phone_number and not self.has_valid_phone_number():
                errors.append(('contact_phone_number', 'bad'))
        return errors

    def has_valid_phone_number(self):
        return is_valid_phone_number(self.contact_phone_number)

    def error_message(self):
        msg = ""
        for errors in self.check_complete():
            msg = msg + "%s is %s, " % (
                " ".join([word.capitalize() for word in errors[0].split("_")]),
                errors[1])
        return (msg and msg[:-2] + '.') or msg

    def set_contact_name(self, contact_name):
        """(contact_name:str)
        """
        typecheck(contact_name, str, allow_none=1)
        self.contact_name = contact_name

    def get_contact_name(self):
        return self.contact_name
    
    def set_contact_phone_number(self, contact_phone_number):
        """(contact_phone_number:str)
        """
        typecheck(contact_phone_number, str, allow_none=1)
        self.contact_phone_number = contact_phone_number

    def get_contact_phone_number(self):
        return self.contact_phone_number
    
    def set_company_name(self, company_name):
        """(company_name:str)
        """
        typecheck(company_name, str, allow_none=1)
        self.company_name = company_name

    def set_street(self, street1, street2):
        """(street1:str, street2:str)
        """
        typecheck(street1, str, allow_none=1)
        typecheck(street2, str, allow_none=1)
        self.street1 = street1
        self.street2 = street2

    def set_city(self, city):
        """(city:str)
        """
        typecheck(city, str, allow_none=1)
        self.city = city

    def set_state(self, state):
        """(state:str)
        """
        typecheck(state, str, allow_none=1)
        self.state = state

    def set_country_code(self, country_code):
        """(country_code:str)
        """
        if country_code is not None:
            typecheck(country_code, str)
            country_code = country_code.upper()
            if not get_country_name(country_code):
                raise ValueError("invalid country code: %r." % country_code)
        self.country_code = country_code

    def set_zip(self, zip):
        """(zip:str)
        """
        typecheck(zip, str, allow_none=1)
        self.zip = zip

    def format(self):
        if self.contact_name:
            address = self.contact_name + "\n"
        else:
            address = ""
        if self.company_name:
            address += self.company_name + "\n"
        if self.street1:
            address += self.street1 + "\n"
        if self.street2:
            address += self.street2 + "\n"
        if self.city:
            address += self.city + ", "
        if self.state:
            address += self.state + " "
        if self.zip:
            address += self.zip
        if address[-1:] != "\n":
            address += "\n"
        if self.country_code:
            address += self.country_code
        return address


class Addressable:
    """
    Mixin for objects that contain addresses

    Instance attributes:
      address : Address
    """

    address_is = nspec(Address)

    def __init__(self):
        self.address = None

    def set_address(self, address):
        """(address : Address)
        """
        typecheck(address, Address)
        self.address = address

    def get_address(self):
        """() -> Address
        """
        return self.address




#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_typeutils.py $
$Id: test_typeutils.py 20909 2003-03-04 14:35:33Z dbinger $

Test for dulcinea.typeutils.
"""

from types import *
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.typeutils import typecheck, typecheck_seq

tested_modules = [ "dulcinea.typeutils" ]

class TypeUtilsTest (TestScenario):

    def check_typecheck_seq (self):
        self.test_stmt("typecheck_seq([4, 2], IntType)")
        self.test_exc("typecheck_seq([4.0, 2], IntType)", TypeError)
        self.test_exc("typecheck_seq([4.0, 2], FloatType)", TypeError)
        self.test_stmt("typecheck_seq([4.0, 2.0], FloatType)")
        self.test_stmt("typecheck_seq(['foo', 'bar', 'baz'], str)")
        self.test_exc("typecheck_seq(['foo', 'bar', 666], str)", TypeError)
        self.test_stmt("typecheck_seq([], str)")
        self.test_stmt("typecheck_seq([], FloatType)")
        self.test_stmt("typecheck_seq([], IntType)")
        self.test_exc("typecheck_seq(4, IntType)", TypeError)
        self.test_exc("typecheck_seq(None, IntType)", TypeError)
        self.test_stmt("typecheck_seq(None, IntType, allow_none=1)")

    def check_typecheck (self):
        self.test_stmt("typecheck(4, IntType)")
        self.test_exc("typecheck(4.0, IntType, allow_none=1)", TypeError)
        self.test_stmt("typecheck(4.0, (FloatType, IntType))")
        self.test_exc("typecheck('foo', (FloatType, IntType))", TypeError)
        self.test_exc("typecheck(None, IntType)", TypeError)
        self.test_stmt("typecheck(None, IntType, allow_none=1)")

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)



#!/www/python/bin/python
"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_item.py $
$Id: test_item.py 23142 2003-11-11 21:18:02Z dbinger $
"""

from mx.DateTime import DateTime
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.item import Item, ItemFolder

class MyItem(Item):
    approved = True

tested_modules = [ "dulcinea.item" ]

class TestItem(TestScenario):
    def setup(self):
        self.item1 = MyItem(timestamp=DateTime(2000))
        self.item1a = MyItem(timestamp=DateTime(2000))
        self.item2 = MyItem(timestamp=DateTime(2002))
        self.item3 = MyItem(timestamp=DateTime(2003))
        self.folder = ItemFolder()

    def check_basic(self):
        self.test_stmt('self.folder.add_item(self.item1)')
        self.test_stmt('self.folder.add_item(self.item2)')
        self.test_set('self.folder.get_items()', [self.item1, self.item2])
        self.test_seq('self.folder.get_recent_items(2)',
                      [self.item2, self.item1])
        self.test_stmt('self.folder.add_item(self.item3)')
        self.test_seq('self.folder.get_recent_items(2)',
                      [self.item3, self.item2])
        self.test_seq('self.folder.get_recent_items(3)',
                      [self.item3, self.item2, self.item1])
        self.test_seq('self.folder.get_recent_items(None)',
                      [self.item3, self.item2, self.item1])
        self.test_stmt('self.folder.add_item(self.item1a)')
        self.test_seq('self.folder.get_recent_items(2)',
                      [self.item3, self.item2])

    def check_delete(self):
        self.test_stmt('self.folder.add_item(self.item1)')
        self.test_stmt('self.folder.add_item(self.item1a)')
        item = MyItem() # auto timestamp
        self.test_stmt('self.folder.add_item(item)')
        self.test_set('self.folder.get_items()',
                      [self.item1, self.item1a, item])
        self.test_stmt('self.folder.delete_item(self.item1a)')
        self.test_set('self.folder.get_items()', [self.item1, item])
 

if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_dumper.py $
$Id: test_dumper.py 20909 2003-03-04 14:35:33Z dbinger $

Test for dulcinea.dumper.
"""

import sys, StringIO
from BTrees import IOBTree
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.dumper import dumps, dump

tested_modules = ["dulcinea.dumper"]

class DumperTest (TestScenario):

    def check_simple (self):
        self.test_val('dumps(1)', '1\n')
        self.test_val('dumps(1.2)', '1.2\n')
        self.test_val('dumps("abcd\\n\\xff")', "'abcd\\n\\xff'\n")

    def check_objects (self):
        # Checks objects that mention their ID, so test_val() isn't used;
        # instead this test case simply verifies that dumps() doesn't crash.
        self.test_stmt('dumps({1:2, 3:4, 5:(6,7)})')
        self.test_stmt('dumps((3, 5, "hello"))')
        self.test_stmt('dumps([3, 5, "hello", [1, 4.5]])')
        self.test_stmt('dumps( IOBTree.IOBTree({1:2, 4:5}))')

    def check_recursive (self):
        # Make a set of recursive objects
        d = {}
        L = [1,2,3, d]
        L.append(L)
        d[1] = L
        d[2] = d
        self.test_stmt('dumps(L)')

    def check_deep (self):
        # Make a deeply nested object
        L = []
        for i in range(50):
            L = [L]
        self.test_stmt('dumps(L)')

    def check_instance (self):
        class C:
            pass
        self.test_stmt('dumps(C())')


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios(scenarios, options)



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/tools/zodb_referrers.py $
$Id: zodb_referrers.py 22951 2003-11-03 00:08:31Z dbinger $

Find and print references to a ZODB object specified by OID.
"""

from dulcinea.script_util import Option, OptionParser, run
from dulcinea.database import pack_oid, format_oid

import sys
import ZODB
import cPickle
import re

def create_index(db):
    index_db = {} # { oid : [refers] }
    storage = db.get_storage()
    # this is fast but may return false positives (i.e. it may find references
    # that don't exist).  It can also return false negatives if the DB has
    # used up more than 4e9 OIDs.
    oid_pat = re.compile(r"U\x08(\x00\x00\x00\x00....)")
    for oid in storage._index.keys():
        pickle_data = storage.load(oid, '')[0]
        for r_oid in oid_pat.findall(pickle_data):
            oids = index_db.setdefault(r_oid, [])
            if oid not in oids:
                oids.append(oid)
    return index_db


def main(db, default_dbspec):

    usage = "usage: %prog [options] [index file]"
    option_list = [
        Option("-c", "--create-index",
               action="store_true", dest="create_index",
               help="create the index file before proceeding"),
        Option("-d", "--database",
               type="string", dest="database",
               help="database specification (e.g. file:db.fs)"),
        ]
    parser = OptionParser(usage, option_list)
    (options, args) = parser.parse_args()

    if len(args) != 1:
        parser.error("you must provide one argument")

    db.open(options.database or default_dbspec)

    if options.create_index:
        if not hasattr(db.get_storage(), "_index"):
            raise SystemExit, "must use FileStorage when creating index"
        index_db = create_index(db)
        cPickle.dump(index_db, open(args[0], "w"), 1)
    else:
        try:
            index_file = open(args[0])
        except IOError:
            parser.error("index file does not exist -- run with -c to create")
        index_db = cPickle.load(index_file)


    connection = db.get_connection()

    while 1:
        try:
            input = raw_input("Enter an OID (or blank line to quit): ").strip()
        except EOFError:
            break
        if not input:
            break
        try:
            if input.startswith("00"):
                # DulcineaBase.__repr__ format
                oid = pack_oid(long(input, 16))
            elif input[0] in "'\"":
                # packed OID as quoted string
                oid = eval(input)
            else:
                raise ValueError
        except ValueError:
            print "Invalid OID: %r" % input
            continue

        def format_object(object):
            name = " " + object.__class__.__name__
            if hasattr(object, "id"):
                name += " (%s)" % object.id
            return "%s%s" % (format_oid(object._p_oid), name)

        seen = {} # {oid : 1}

        def report_references(oids, indent='  '):
            for oid in oids:
                if oid not in seen:
                    seen[oid] = 1
                    object = connection[oid]
                    sys.stdout.write("%s%s\n" % (indent, format_object(object)))
                    #report_references(index_db.get(oid, []), indent+'  ')
        print format_object(connection[oid])
        report_references(index_db.get(oid, []))


if __name__ == "__main__":
    run()



#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_history.py $
$Id: test_history.py 23447 2004-02-06 17:36:45Z dbinger $

Test for dulcinea.history.
"""

import re, time
from mx.DateTime import DateTime
from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.user import DulcineaUser
from dulcinea.history import History, Event, Historical

tested_modules = [ "dulcinea.history" ]

class HistoryTest (TestScenario):

    def setup(self):
        self.user1 = DulcineaUser("joe")
        self.user2 = DulcineaUser("jane")

    def shutdown(self):
        del self.user1, self.user2

    def check_historical(self):
        self.test_stmt("Historical().get_history()")

    def check_event(self):
        user = DulcineaUser("jblow")
        message = "created new foobar ding-dong"
        event1 = Event(user, "create", message)
        self.test_val("event1.get_user()", user)
        self.test_val("event1.get_event_code()", 'create')
        self.test_val("event1.get_message()", message)

        self.test_val("str(event1)", "%s create" % event1.timestamp)

        event2 = Event(user, "modify", "changed blah blah on foobar")
        self.test_true("event2.timestamp >= event1.timestamp")
        time.sleep(0.01)
        event3 = Event(user, "delete", "jblow deleted foobar")
        self.test_true("event3.timestamp > event2.timestamp")
        self.test_true("event3.timestamp is event3.get_timestamp()")


    def check_cmp(self):
        user = DulcineaUser("jblow")
        t1 = DateTime(1999, 12, 1, 15, 34, 2)
        t2 = DateTime(1999, 12, 1, 15, 34, 2.01)
        t3 = DateTime(2001, 5, 19, 8, 30)

        event1 = Event(user, "create", "created new thing", t1)
        event2 = Event(user, "create", "created new thing", t1)
        event3 = Event(user, "create", "created new things", t1)
        event4 = Event(user, "create", "created new things", t2)
        event5 = Event(user, "create", "created new thing", t2)
        event6 = Event(user, "edit", "edited a thing", t1-1)

        self.test_true("event1 == event1")
        self.test_true("event1 == event2")
        self.test_true("event1 != event3")   # because of message
        self.test_true("event1 < event3")
        self.test_true("event3 > event1")
        self.test_true("event2 < event4")    # because of timestamp
        self.test_true("event4 >= event2")
        self.test_true("event6 < event1")
        self.test_true("event5 > event6")
        event4.timestamp = None
        self.test_true("event2 > event4")    # because of timestamp
        self.test_true("event4 < event2")    # because of timestamp

    def check_history(self):
        history = History()
        self.test_val("len(history)", 0)
        self.test_val("str(history)", "history (0 events)")
        self.test_stmt("history.add_event(self.user1, 'create', "
                                         "'joe created blah')")
        self.test_stmt("history.add_event(None, 'modify', "
                                         "'jane modified blah')")
        self.test_val("len(history)", 2)
        self.test_true("history[0].timestamp <= history[1].timestamp")
        self.test_val("history[0].user", self.user1)
        self.test_val("history[1].user", None)

        ts = DateTime(0)
        self.test_stmt("history.add_event(self.user2, 'create',"
                                         "'god created the universe',"
                                         "timestamp=ts)")
        self.test_val("len(history)", 3)
        self.test_true("history[0].timestamp < "
                       "history[1].timestamp <= "
                       "history[2].timestamp")
        self.test_val("history[0].message", "god created the universe")

        ts = DateTime(1500)
        self.test_stmt("history.add_event(None, 'create', "
                                         "'modern age begins', "
                                         "timestamp=ts)")
        self.test_val("len(history)", 4)
        self.test_val("history[0].message", "god created the universe")
        self.test_val("history[1].message", "modern age begins")


    def check_variations(self):
        history = History()
        # Works with no user object
        self.test_stmt("history.add_event(None, 'modify', 'tweaked it')")
        self.test_val("history[-1].user", None)

        # Replacing existing event
        self.test_stmt("history.add_unique_event(None, 'modify', 'tweaked it more')")
        self.test_val("history[-1].message", 'tweaked it more')

        self.test_stmt("history.add_unique_event(None, 'create', 'creation')")
        self.test_val("history[-2].message", 'tweaked it more')

    def check_find(self):
        history = History()
        self.test_val("history.find_event()", None)
        self.test_val("history.find_event(latest=True)", None)

        history.add_event(self.user1, "create", "joe created it")
        self.test_val("history.find_event().message",
                      "joe created it")
        self.test_val("history.find_event(latest=True).message",
                      "joe created it")
        self.test_val("history.find_event(user=self.user1).message",
                      "joe created it")
        self.test_val("history.find_event(user=self.user2)", None)
        self.test_val("history.find_event(event_code='create').message",
                      "joe created it")
        self.test_val("history.find_event(message='no created it')", None)

        self.test_val("history.find_event(event_code='access')", None)
        self.test_val("history.find_event(event_codes=['access'])", None)

        history.add_event(self.user2, "access", "jane accessed it")
        history.add_event(self.user1, "modify", "joe modified it")
        history.add_event(self.user2, "modify", "jane modified it")
        history.add_event(self.user2, "delete", "jane deleted it")

        self.test_val("history.find_event().message",
                      "joe created it")
        self.test_val("history.find_event(latest=True).message",
                      "jane deleted it")
        self.test_val("history.find_event(user=self.user1,latest=True)"
                      ".message",
                      "joe modified it")
        self.test_val("history.find_event(event_code='modify').message",
                      "joe modified it")
        self.test_val(
            "history.find_event(event_code='modify',latest=True).message",
            "jane modified it")
        self.test_val("len(history.find_events_after(0))", 5)
        self.test_val("history.find_last_date('foo')", None)


if __name__ == "__main__":
    (scenarios, options) = parse_args()
    run_scenarios (scenarios, options)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/dumper.py $
$Id: dumper.py 22988 2003-11-05 01:56:52Z dbinger $

Dump Python data structures (including class instances) in a nicely-
nested, easy-to-read form.  Handles recursive data structures properly,
and has sensible options for limiting the extent of the dump both by
simple depth and by some rules for how to handle contained instances.
"""

import sys, string
from StringIO import StringIO
from types import *

DICT_TYPES = {DictionaryType: 1}
try:
    from BTree import BTree
    DICT_TYPES[BTree] = 1
except ImportError:
    pass
try:
    from BTrees import OOBTree, OIBTree, IOBTree
    DICT_TYPES[OOBTree.OOBTree] = 1
    DICT_TYPES[OIBTree.OIBTree] = 1
    DICT_TYPES[IOBTree.IOBTree] = 1
except ImportError:
    pass

#
# IDEAS on how to restrict how deep we go when dumping:
#   - never follow cyclic links! (handled with 'seen' hash)
#   - have a maximum dumping depth (for any nestable structure:
#     list/tuple, dictionary, instance)
#   - if in an instance: don't dump any other instances
#   - if in an instance: don't dump instances that belong to classes from
#     other packages
#   - if in an instance: don't dump instances that belong to classes from
#     other modules
# ...ok, these are all implemented now -- cool!

# restrictions on dumping
max_depth = 5
instance_dump = 'module'                # or 'all', 'none', 'package':
                                        # controls whether we dump an
                                        # instance that is under another
                                        # instance (however deep)

# arg -- this is necessary because the .__name__ of a type object
# under JPython is a bit ugly (eg. 'org.python.core.PyList' not 'list')
TYPE_NAMES = {
    BuiltinFunctionType: 'builtin',
    BuiltinMethodType: 'builtin',
    ClassType: 'class',
    CodeType: 'code',
    ComplexType: 'complex',
    DictType: 'dictionary',
    DictionaryType: 'dictionary',
    EllipsisType: 'ellipsis',
    FileType: 'file',
    FloatType: 'float',
    FrameType: 'frame',
    FunctionType: 'function',
    InstanceType: 'instance',
    IntType: 'int',
    LambdaType: 'function',
    ListType: 'list',
    LongType: 'long int',
    MethodType: 'instance method',
    ModuleType: 'module',
    NoneType: 'None',
    SliceType: 'slice',
    StringType: 'string',
    TracebackType: 'traceback',
    TupleType: 'tuple',
    TypeType: 'type',
    UnboundMethodType: 'instance method',
    UnicodeType: 'unicode',
    XRangeType: 'xrange',
    }

def get_type_name(type):
    try:
        return TYPE_NAMES[type]
    except KeyError:
        return type.__name__

def is_instance(val):
    if type(val) is InstanceType:
        return 1
    # instance of extension class, but not an actual extension class
    elif (hasattr(val, '__class__') and
          hasattr(val, '__dict__') and
          not hasattr(val, '__bases__')):
        return 1
    else:
        return 0

def is_class(val):
    return hasattr(val, '__bases__')

def atomic_type(t):
    return t in (NoneType, StringType, IntType, LongType, FloatType,
                 ComplexType)

def short_value(val):
    #if atomic_type(type(val)):
    #    return 1
    t = type(val)
    if (not DICT_TYPES.has_key(t) and t not in (ListType, TupleType) and
            not is_instance(val)):
        return 1
    elif t in (ListType, TupleType) and len(val) <= 10:
        for x in val:
            if not atomic_type(type(x)):
                return 0
        return 1
    elif DICT_TYPES.has_key(t) and len(val) <= 5:
        for (k,v) in val.items():
            if not (atomic_type(type(k)) and atomic_type(type(v))):
                return 0
        return 1
    else:
        return 0


def object_summary(val):
    t = type(val)
    if is_instance(val):
        if hasattr(val, '__str__'):
            strform = ": " + str(val)
        else:
            strform = ""
        return "<%s at %x%s>" % (val.__class__.__name__, id(val), strform)
    elif is_class(val):
        return "<%s %s at 0x%x>" % (get_type_name(t), val.__name__, id(val))
    else:
        return "<%s at 0x%x>" % (get_type_name(t), id(val))

def short_dump(val):
    if atomic_type(type(val)) or is_instance(val) or is_class(val):
        return `val`
    else:
        return object_summary(val) + ': ' + `val`

class Dumper:

    def __init__(self, max_depth=None, instance_dump=None, output=None):
        self._max_depth = max_depth
        self._instance_dump = instance_dump
        if output is None:
            self.out = sys.stdout
        else:
            self.out = output

    def __getattr__(self, attr):
        if self.__dict__.has_key('_' + attr):
            val = self.__dict__['_' + attr]
            if val is None:             # not defined in object;
                # attribute exists in instance (after adding _), but
                # not defined: get it from the module globals
                globals = vars(sys.modules[__name__])
                return globals.get(attr)
            else:
                # _ + attr exists and is defined (non-None)
                return val
        else:
            # _ + attr doesn't exist at all
            raise AttributeError, attr

    def __setattr__(self, attr, val):
        if self.__dict__.has_key('_' + attr):
            self.__dict__['_' + attr] = val
        else:
            self.__dict__[attr] = val

    def _writeln(self, line):
        self.out.write(line + "\n")

    def dump(self, val, indent='', summarize=1):
        self.seen = {}
        self.containing_instance = []
        self._dump(val, indent=indent, summarize=summarize)

    def _dump(self, val, depth=0, indent='', summarize=1):
        t = type(val)
        if short_value(val):
            self._writeln("%s%s" % (indent, short_dump(val)))
        else:
            depth = depth + 1
            if depth > self.max_depth:
                self._writeln(indent + "contents suppressed (too deep)")
                return
            if self.seen.get(id(val)):
                self._writeln(indent + "object already seen")
                return
            self.seen[id(val)] = 1
            if DICT_TYPES.has_key(t):
                if summarize:
                    self._writeln("%s%s:" % (indent, object_summary(val)))
                    indent = indent + '  '
                self.dump_dict(val, depth, indent)
            elif t in (ListType, TupleType):
                if summarize:
                    self._writeln("%s%s:" % (indent, object_summary(val)))
                    indent = indent + '  '
                self.dump_sequence(val, depth, indent)
            elif is_instance(val):
                self.dump_instance(val, depth, indent, summarize)
            else:
                raise RuntimeError, "this should not happen"

    def dump_dict(self, dict, depth, indent, shallow_attrs=()):
        keys = dict.keys()
        if type(keys) is ListType:
            keys.sort()
        for k in keys:
            val = dict[k]
            if short_value(val) or k in shallow_attrs:
                self._writeln("%s%s: %s" % (indent, k, short_dump(val)))
            else:
                self._writeln("%s%s: %s" % (indent, k, object_summary(val)))
                self._dump(val, depth, indent+'  ', summarize=0)

    def dump_sequence(self, seq, depth, indent):
        for i in range(len(seq)):
            val = seq[i]
            if short_value(val):
                self._writeln("%s%d: %s" % (indent, i, short_dump(val)))
            else:
                self._writeln("%s%d: %s" % (indent, i, object_summary(val)))
                self._dump(val, depth, indent+'  ', summarize=0)


    def dump_instance(self, inst, depth, indent, summarize=1):
        if summarize:
            self._writeln(indent + "%s " % object_summary(inst))
            indent = indent + '  '
        instance_dump = self.instance_dump
        # already dumping a containing instance, and have some restrictions
        # on instance-dumping?
        if self.containing_instance and instance_dump != 'all':
            previous_instance = self.containing_instance[-1]
            container_module = previous_instance.__class__.__module__
            container_package = (string.split(container_module, '.'))[0:-1]
            current_module = inst.__class__.__module__
            current_package = (string.split(current_module, '.'))[0:-1]
            # inhibit dumping of all contained instances?
            if instance_dump == 'none':
                return
            # inhibit dumping instances from a different package?
            elif (instance_dump == 'package' and
                  current_package != container_package):
                return
            # inhibit dumping instances from a different module?
            elif (instance_dump == 'module' and
                  current_module != container_module):
                return
        # if in containing instance and have restrictions
        self.containing_instance.append(inst)
        shallow_attrs = getattr(inst, "_dump_shallow_attrs", [])
        self.dump_dict(vars(inst), depth, indent, shallow_attrs)
        del self.containing_instance[-1]

default_dumper = Dumper()

def dump(val, output=None):
    if output is None:
        default_dumper.dump(val)
    else:
        Dumper(output=output).dump(val)

def dumps(val):
    out = StringIO()
    Dumper(output=out).dump(val)
    return out.getvalue()


if __name__ == "__main__":
    l1 = [3, 5, 'hello']
    t1 = ('uh', 'oh')
    l2 = ['foo', t1]
    d1 = {'k1': 'val1',
          'k2': l1,
          'k2': l2}

    dump(l1)
    dump(t1)
    dump(l2)
    dump(d1)

    dumper = Dumper(max_depth=1)
    l = ['foo', ['bar', 'baz', (1, 2, 3)]]
    dumper.dump(l)
    dumper.max_depth = 2
    dumper.dump(l)
    l[1][2] = tuple(range(11))
    dumper.dump(l)
    dumper.max_depth = None
    print dumper.max_depth

    class Foo: pass
    class Bar: pass

    f = Foo()
    b = Bar()

    f.a1 = 35
    f.a2 = l1
    f.a3 = l2
    f.b = b
    f.a4 = l2
    b.a1 = f
    b.a2 = None
    b.a3 = 'foo'
    dump(f)



'''$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/__init__.py $
$Id: __init__.py 22753 2003-10-15 21:26:17Z nascheme $
'''

from dulcinea.site_util import LazyModule as _LazyModule

# import these from here but do not access their attributes at the
# module-level.  That will lead to circular import problems.
local = _LazyModule('local')
local_ui = _LazyModule('local_ui')



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/typed_list.py $
$Id: typed_list.py 23443 2004-02-06 15:23:48Z dbinger $

Provides TypedList.
"""

from dulcinea.typeutils import typecheck
from dulcinea.base import DulcineaPersistent

class TypedList (DulcineaPersistent):
    """A list of named objects with find/add/remove operations.

    Instance attributes:
      list : [any]
        the actual list of objects
    """

    required_class = None
    nameattr = "name"

    # -- Initialize/overload methods -----------------------------------

    def __init__(self, values=None, check_types=1):
        # Note that list is not persistent.
        if values:
            if check_types:
                self.set_list(values)
            else:
                self.list = list(values)
        else:
            self.list = []

    def copy(self):
        return self.__class__(self.list, check_types=0)

    def _check_value(self, value):
        """Check that 'value' as an instance of the required class"""
        if self.required_class is not None:
            typecheck(value, self.required_class)

    def _check_name(self, name):
        if self.find(name) is not None:
            raise ValueError, \
                  "list already has a value named '%s'" % name

    def _get_name(self, value):
        if hasattr(value, self.nameattr):
            return getattr(value, self.nameattr)
        return getattr(value, 'get_%s' % self.nameattr)()

    def __str__(self):
        np = len(self.list)
        if np == 0:
            return "no values"
        elif np == 1:
            return "1 value"
        else:
            return "%d values" % np

    def __getitem__(self, i):
        return self.list[i]

    def __setitem__(self, i, value):
        self._check_value(value)
        old_name = self._get_name(self.list[i])
        new_name = self._get_name(value)
        if old_name != new_name:
            self._check_name(new_name)
        self.list[i] = value
        self._p_changed = 1

    def __len__(self):
        return len(self.list)

    def __add__(self, other):
        newlist = self.__class__()
        if isinstance(other, TypedList):
            newlist.list = self.list + other.list
        else:
            newlist.list = self.list + other
        return newlist

    # -- Accessor methods ----------------------------------------------

    def get_list(self):
        """() -> [any]
        Return the actual list of values buried in the list.  This is not a
        copy, so be careful what you do with it.  If you do update the list,
        make sure that you only add instances of the proper type to it.
        """
        return self.list

    def get_names(self):
        """() -> [string]

        Return the list of all names in the list.
        """
        return [ self._get_name(v) for v in self.list ]


    # -- Modifier methods ----------------------------------------------

    def set_list(self, values):
        """(values : [any])

        Set the actual list of values.
        Raises TypeError if any element of 'values' is invalid.
        Raises ValueError on name collisions.
        """
        self.list = []
        for value in values:
            self.add(value)


    # -- Search/update methods -----------------------------------------

    def find(self, name):
        """(name : string) -> any

        Searches the list for a value with name 'name'.  Returns the
        matching value, or None if it couldn't find one.
        """
        typecheck(name, str)
        for object in self.list:
            if name == self._get_name(object):
                return object
        return None

    def add(self, value):
        """(value : any)

        Add the value 'value' (which must be of type required_type).  Raise
        TypeError if 'value' is not of the required class; raise ValueError
        if this list already has a value with the same name.
        """
        self._check_value(value)
        self._check_name(self._get_name(value))
        self.list.append(value)
        self._p_changed = 1

    def merge(self, other, resolve='error'):
        """Add all the values from another list ('other') to the current
        list.  Any name conflicts between the two lists are resolved
        according to 'resolve': if it is "error", then any name conflict
        triggers a ValueError exception; if it is "replace", then the
        conflicting value in the current exception list is replaced with its
        same-named counterpart from 'other'; if it is 'ignore', the
        conflicting value from 'other' is ignored.

        Raise TypeError if 'other' is not a TypedList; raise ValueError if
        there are name conflicts between the two lists and 'resolve' is
        "error"."""

        assert (resolve in ['error', 'replace', 'ignore'])

        typecheck(other, TypedList)

        # Build a dictionary mapping name -> list index for the
        # current list -- makes it easy to detect conflicts
        self_values = {}
        for i in range(len(self.list)):
            self_values[self._get_name(self.list[i])] = i

        # Now walk over the 'other' list -- the prospective list of
        # additions to 'self'.  For non-conflicting values, it's a
        # doddle, just add 'em to the end of self's list.  Values
        # with name conflicts must be dealt with a bit more
        # intelligently, depending on the value of
        # 'resolve'.

        for o_value in other:
            o_name = self._get_name(o_value)
            conflict = self_values.get(o_name)
            if conflict is not None:
                if resolve == 'error':
                    raise ValueError, \
                          "name conflict between lists: '%s'" % o_name
                # replace same-named values in current list with
                # other's version
                elif resolve == 'replace':
                    self.list[conflict] = o_value

                # ignore other's version, keep current list's version
                elif resolve == 'ignore':
                    pass

            # No conflict -- no problem
            else:
                self.list.append (o_value)


    def remove(self, name):
        """(name : string)

        Find the value named by 'name', remove and return it.  If no such
        value was found, does nothing and returns None.
        """
        for i in range (len (self.list)):
            if self._get_name(self.list[i]) == name:
                goner = self.list[i]
                del self.list[i]
                self._p_changed = 1
                return goner
        else:
            return None

    def find_index(self, name):
        """(name : string)

        Find the value named by 'name', and return its index.
        If no such value was found, returns None.
        """
        for i in range(len(self.list)):
            if self._get_name(self.list[i]) == name:
                return i
        else:
            return None



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/user_database.py $
$Id: user_database.py 23268 2003-12-04 12:26:43Z dbinger $

Provides the UserDatabase class.
"""

from dulcinea.base import DulcineaPersistent, PersistentMapping
from dulcinea.sort import lexical_sort
from dulcinea.errors import ConflictError
from dulcinea.permission import PermissionManager
from dulcinea.user import Contact

class DulcineaUserDatabase(DulcineaPersistent):
    """
    Class to hold all users in the system.  User IDs are always looked
    up in the user database, so you will generally not be able to use
    a user until it has been added to the user database.

    Instance attributes:
      users : PersistentMapping { user_id:string : user:DulcineaUser }
        all known users
    """

    def __init__ (self):
        self.users =  PersistentMapping()

    def get_user_ids (self):
        return self.users.keys()

    def get_users (self, sort=0):
        users = self.users.values()
        if sort:
            users = lexical_sort(users)
        return users

    def get_user (self, user_id):
        """Return the User object with id 'user_id', or None if no such
        user."""
        return self.users.get(user_id)

    def __getitem__ (self, user_id):
        return self.users[user_id]

    def add_user (self, user):
        """Add User object 'user' to the user database.  Raises
        ConflictError if there is already a user with the same user
        ID.
        """
        if self.users.has_key(user.id):
            raise ConflictError, \
                  "user '%s' already exists in the user database" % \
                  user.id
        self.users[user.id] = user

    def get_admin(self):
        """Subclasses should override to return a PermissionManager
        """
        return None


class ContactDatabase(DulcineaUserDatabase):
    """
    Instance attributes:
      admin: PermissionManager
    """
    def __init__(self):
        DulcineaUserDatabase.__init__(self)
        system_user = Contact("SYSTEM")
        self.add_user(system_user)
        self.admin = ContactAdmin()
        self.admin.set_permission('system', system_user)

    def get_admin(self):
        return self.admin

class ContactAdmin(DulcineaPersistent, PermissionManager):
    """
    Instance attributes: none
    """
    valid_permissions = {
        "manage-permissions":
            "Allow changing of permissions.",
        "staff":
            ("Is a member of the staff, with all of the privileges and "
             "responsibilities thereunto appertaining."),
        "act-as":
            "Allow to act as another user.",
        "system":
            "Allow to do things normally done by the software system."
        }

    def __init__(self):
        PermissionManager.__init__(self)

    def __str__(self):
        return 'admin'





#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/test_spec.py $
$Id: test_spec.py 23442 2004-02-05 22:32:06Z dbinger $

Test for dulcinea.spec.
"""

from sancho.unittest import TestScenario, parse_args, run_scenarios
from dulcinea.base import DulcineaPersistent
from dulcinea.spec import spec, nspec, any, get_spec_doc, get_specs, \
     get_spec_problems, match, set, require, format_spec, interval, \
     string, sequence, eq, anything, mapping, equal, subclass, instance, \
     all, length, no, boolean, get_spec, get_spec_report

tested_modules = [ "dulcinea.spec" ]

class SpecTest (TestScenario):

    def check_match (self):
        self.test_bool("match(1, int)")
        self.test_bool("not match(2.5, int)")
        self.test_bool("match(2.5, any(int, float))")
        self.test_bool("not match('s', any(int, float))")
        self.test_bool("not match('hello', sequence(int, str))")
        self.test_bool("match('hello', sequence(str, str))")
        self.test_bool("not match('hello', sequence(str, tuple))")
        self.test_bool("match('hello', str)")
        self.test_bool("match('hello', any('yes', 'no', 'hello', int))")
        self.test_bool("match(3, any('yes', 'no', 'hello', int))")
        mydict = {'a': 1, 'b': None}
        self.test_bool("match(mydict, dict)")
        self.test_bool("match(mydict, { str : object })")
        self.test_bool("not match(mydict, { int : object })")
        self.test_bool("match(mydict, { str : object, int : object })")
        self.test_bool("match((2,3), (int, int))")
        self.test_bool("match((1,), tuple)")
        self.test_bool("match((1,), sequence(object, tuple))")
        self.test_bool("match((1,2,3,4), sequence(object, tuple))")
        self.test_bool("match((1,2,3,'ok'), sequence(object, tuple))")
        self.test_bool("not match((1,2,3,'ok'), sequence(int, tuple))")
        self.test_bool("match(mydict, mapping({ any(str, int) : object }))")
        self.test_bool("not match(mydict, mapping({int:object}, dict))")
        self.test_bool("not match(mydict, mapping({str: int}))")
        self.test_bool("match(True, boolean)")
        self.test_bool("not match(3, boolean)")
        self.test_bool("match([3,'ok'], [int, str])")
        self.test_bool("not match([3,'ok', 3], [int, str])")
        self.test_bool("match(3, no(None))")
        self.test_bool("match([3, 4], [all(int, no(None))])")
        self.test_bool("not match(3, any)")
        self.test_bool("match(3, object)")
        a=[2,3]
        b=[2,3]
        self.test_bool("match(a, eq(a))")
        self.test_bool("not match(b, eq(a))")
        self.test_bool("match(a, equal(b))")
        self.test_bool("match('a', string(1))")
        self.test_bool("match('a', length(1))")
        self.test_bool("match('a', interval('a','c'))")
        self.test_bool("not match('b', all(length(1), string(2)))")
        self.test_bool("not match('b', all(length(2)))")
        self.test_bool("match(10, interval(1))")
        self.test_bool("match(1, interval(1))")
        self.test_false("match(0, interval(1))")
        self.test_false("match(0, interval(1,4))")
        self.test_true("match(4, interval(1,4))")
        self.test_false("match(5, interval(1,4))")
        self.test_true("match(0, interval(anything,4))")
        self.test_true("match(0, instance('int'))")
        self.test_false("match(0, no(int))")

        class foo (dict, TestScenario):
            pass
        self.test_false("match(0, subclass(dict))")
        self.test_true("match(foo, subclass(dict))")
        self.test_false("match(0, mapping())")
        self.test_false("match(0, mapping({int:int},foo))")
        self.test_stmt("sequence().get_args()")
        self.test_false("match(0, instance('foo'))")
        for s in ('equal(1)',
                  'eq(1)',
                  'string(4)',
                  'sequence(2, anything)',
                  'interval(2, 4)',
                  'interval(2)',
                  'interval()',
                  'interval(anything, 3)',
                  'None',
                  '(int, str)',
                  '[int]',
                  '{int: str}',
                  "'constant'",
                  ):
            self.test_val("format_spec(%s)" % s, s)
        self.test_exc("require(1, str)", TypeError)
        self.test_exc("require(1, str, 'oops')", TypeError)
        self.test_false("match(1, [1])")
        self.test_false("match([1], [str])")
        self.test_false("match([1, 'hello'], [str, str])")
        self.test_false("match(1, {})")
        class mytest:
            def __call__ (self, value):
                return True
        self.test_true("match(1, mytest())")

    def check_specified (self):
        class A:
            flavor_is = spec(any('sweet', 'sour'), 'the flavor')
            texture_is = spec(int, 'roughness')

        class B(A):
            color_is = spec(str)
            texture_is = nspec(str)

            
        self.a = A()
        self.b = B()
        self.b.color = self.b.texture = 'ok'
        self.test_stmt("set(self.b, flavor='sweet')")
        self.test_bool("get_spec_problems(self.b) == []")
        self.test_stmt("self.b.nonsense = 34")
        self.test_bool("len(get_spec_problems(self.b)) == 1")
        self.test_stmt("del self.b.nonsense")
        self.test_bool("len(get_spec_problems(self.b)) == 0")
        self.test_stmt("self.b.color = 42")
        self.test_bool("len(get_spec_problems(self.b)) == 1")
        self.test_stmt("self.b.color = 'red'")
        self.test_bool("len(get_spec_problems(self.b)) == 0")
        self.test_bool("len(get_specs(B).values()) == 3")
        def is_texture(value):
            return value in (3,4)
        class C(DulcineaPersistent):
            color_is = nspec(str, 'the color')
            texture_is = is_texture
        self.test_bool("get_specs(C)")
        self.test_bool("get_spec_doc(C)")
        self.test_stmt("nspec(None)")
        self.test_stmt("nspec(int)")
        self.test_stmt("nspec(any(int, str))")
        self.test_stmt("get_spec(C, 'texture_is')")
        class D:
            color_is = nspec(str, 'the color')
            texture_is = is_texture
        self.test_stmt("get_spec(D, 'texture_is')")
        self.test_stmt("get_spec_problems(D())")
        class E:
            pass
        self.test_stmt("get_spec_report([D()])")
        self.test_stmt("get_spec_report([E()])")
        self.test_stmt("get_spec_report([])")
        
if __name__ == "__main__":
    (scenarios, options) = parse_args()
    options.show_coverage_lines = options.show_coverage = 1
    run_scenarios (scenarios, options)
